/*
 * imaverage.c -- Average fits ccd 2D images
 *
 * John Kielkopf
 * kielkopf@louisville.edu
 * 
 *
 * Current Version 1.0 -- June 14, 2007
 *
 *   June 13, 2007 : Version 1.0 : First version from imflat and imbuild
 *   
 */

#include <string.h>
#include <stdio.h>
#include <math.h> 
#include "fitsio.h"

int main(int argc, char *argv[])
{
  fitsfile *afptr, *outfptr;  
  int status = 0;  
  int atype, anaxis, i, j, k;
  long npixels = 1, mpixels = 1, firstpix[2] = {1,1};
  long anaxes[2] = {1,1};

  double *apix, *sumpix;
  double anorm;

  if (argc < 4) { 
    printf("Usage: imaverage out_image in_image1 in_image2 ...\n");
    printf("\n");
    printf("Find an average of several images\n");
    printf("\n");
    printf("Example: \n");
    printf("  imaverage flatav.fit flat1.fit flat2.fit flat3.fit\n");
    return(0);
  }

  /* How many files will be averaged? */
      
  anorm = argc - 2;

  /* Process one input image at a time using a local image buffer */

  for (k = 2; k < argc; k++)
  {
  
    /* Open an image */
  
    printf("Opening image %d/%d \n",k-1,argc-2);
    
    fits_open_file(&afptr, argv[k], READONLY, &status);

    /* Read dimensions */

    fits_get_img_dim(afptr, &anaxis, &status);
    fits_get_img_size(afptr, 2, anaxes, &status);
  
    /* Report errors if any */

    if (status) 
    {
      fits_report_error(stderr, status); 
      return(status);
    }

    if (k == 2)
    {
      /* Number of pixels to read in each row */
  
      npixels = anaxes[0];
      
      /* Number of pixels in a column */
      
      mpixels = anaxes[1];
  
      /* Allocate memory for one input row */ 

      apix = (double *) malloc(npixels * sizeof(double)); 
      
      /* Allocate memory for entire output image */
      
      sumpix = (double *) malloc(npixels * mpixels * sizeof(double)); 
    
      /* Initialize the final image memory to zero */
      
      for ( i = 0 ; i < ( npixels * mpixels ) ; i ++ )
      {
        sumpix[i] = 0;
      }        
    }
    
    
    if (k > 2)
    {
      if ( (anaxes[0] != npixels) || (anaxes[1] != mpixels) )
      {
         printf("  File %d does not have correct size.", k-2);
         fits_close_file(afptr, &status); 
         return(0);          
      }
    }
    
    /* Start with first pixel in each row */
  
    firstpix[0] = 1;
    
    /* Start with first pixel in sum array */
      
    j = 0;

    /* Process images one row at a time */
      
    for (firstpix[1] = 1; firstpix[1] <= anaxes[1]; firstpix[1]++)
    {  
     
      /* Find starting pixel coordinate and number of pixels to read */
     
      if (fits_read_pix(afptr, TDOUBLE, firstpix, npixels,
         NULL, apix, NULL, &status))
      break; 
      
      for (i = 0; i < anaxes[0]; i++) 
      {
        sumpix[j] += apix[i];
        j++;          
      }
    }
    
    /* Close this image */      
    
    fits_close_file(afptr, &status);       

  }
      
  /* Reopen the first input image to get the header info */
  
  fits_open_file(&afptr, argv[2], READONLY, &status);

  /* Create the new empty output file */

  fits_create_file(&outfptr, argv[1], &status);

  /* Copy all the header keywords from first input image */
  
  fits_copy_header(afptr, outfptr, &status);

  /* Loop over rows in output image using the old apix buffer */
  /* Renormalize for average in this loop */
  
  j=0;
    
  for (firstpix[1] = 1; firstpix[1] <= anaxes[1]; firstpix[1]++)
  {
    
    for(i=0; i < npixels; i++)
    {
      apix[i] = sumpix[j];
      apix[i] = apix[i] / anorm ;
      j++;
    }
    
    /* Write new values to output image */

    fits_write_pix(outfptr, TDOUBLE, firstpix, npixels,
                 apix, &status); 
  }
  
  fits_close_file(afptr, &status);    
  fits_close_file(outfptr, &status);
  free(apix);
  free(sumpix);
  
  /* Report any error message */
 
  if (status) fits_report_error(stderr, status); 
  return(status);
}

